/*****************************************************************************
*
* ALPS Project Applications
*
* Copyright (C) 2005 by Matthias Troyer <troyer@comp-phys.org>,
*                       Andreas Streich <astreich@student.ethz.ch>
*
* This software is part of the ALPS Applications, published under the ALPS
* Application License; you can use, redistribute it and/or modify it under
* the terms of the license, either version 1 or (at your option) any later
* version.
* 
* You should have received a copy of the ALPS Application License along with
* the ALPS Applications; see the file LICENSE.txt. If not, the license is also
* available from http://alps.comp-phys.org/.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
* FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT 
* SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE 
* FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE, 
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
* DEALINGS IN THE SOFTWARE.
*
*****************************************************************************/

/* $Id$ */

#include "fitting_scheduler.h"

#include <alps/scheduler/types.h>
#include <alps/osiris/comm.h>

namespace alps {
namespace scheduler {

int start_fitting(int argc, char** argv, const Factory& p) 
{
  if (argc < 2)
    return -1;

  comm_init(argc,argv);

  if ((is_master()) || (!runs_parallel())) {
    p.print_copyright(std::cout);
    alps::scheduler::print_copyright(std::cout);
    alps::print_copyright(std::cout);
  }

  NoJobfileOptions njfo(argc-1,argv);
  if (!(njfo.valid)) {
    std::cerr << "invalid options, returning \n";
    return -1;
  }

  if (!runs_parallel()) 
    theScheduler = new FittingScheduler<SingleScheduler>(njfo,p,argv[argc-1]);
  else if (is_master())
    theScheduler = new FittingScheduler<MPPScheduler>(njfo,p,argv[argc-1]);
  else
    theScheduler = new Scheduler(njfo,p);

  theScheduler->run();

  delete theScheduler;
  comm_exit();
  return 0;
}

} // namespace scheduler
} // namespace alps
